@@ -2,7 +2,7 @@
  * PLUG - software to operate Fender Mustang amplifier
  *        Linux replacement for Fender FUSE software
  *
- * Copyright (C) 2017-2022  offa
+ * Copyright (C) 2017-2023  offa
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -21,6 +21,7 @@
 #pragma once
 
 #include <vector>
+#include <string>
 #include <cstdint>
 
 namespace plug::com
@@ -42,6 +43,8 @@ namespace plug::com
 
         virtual std::vector<std::uint8_t> receive(std::size_t recvSize) = 0;
 
+        virtual std::string name() const = 0;
+
     private:
         virtual std::size_t sendImpl(std::uint8_t* data, std::size_t size) = 0;
     };
