@@ -2,7 +2,7 @@
  * PLUG - software to operate Fender Mustang amplifier
  *        Linux replacement for Fender FUSE software
  *
- * Copyright (C) 2017-2022  offa
+ * Copyright (C) 2017-2023  offa
  * Copyright (C) 2010-2016  piorekf <piorek@piorekf.org>
  *
  * This program is free software: you can redistribute it and/or modify
@@ -37,6 +37,7 @@
 #include "com/MustangUpdater.h"
 #include "ui_defaulteffects.h"
 #include "ui_mainwindow.h"
+#include <algorithm>
 #include <QFileDialog>
 #include <QMessageBox>
 #include <QSettings>
@@ -50,15 +51,25 @@ namespace plug
         constexpr int check_fx_family(effects value)
         {
             if (value == effects::EMPTY)
+            {
                 return 0;
+            }
             if ((value >= effects::OVERDRIVE) && (value <= effects::COMPRESSOR))
+            {
                 return 1;
+            }
             if ((value >= effects::SINE_CHORUS) && (value <= effects::PITCH_SHIFTER))
+            {
                 return 2;
+            }
             if ((value >= effects::MONO_DELAY) && (value <= effects::STEREO_TAPE_DELAY))
+            {
                 return 3;
+            }
             if ((value >= effects::SMALL_HALL_REVERB) && (value <= effects::FENDER_65_SPRING_REVERB))
+            {
                 return 4;
+            }
             return 0;
         }
 
@@ -69,7 +80,15 @@ namespace plug
         : QMainWindow(parent),
           ui(std::make_unique<Ui::MainWindow>()),
           presetNames(100, ""),
-          amp_ops(nullptr)
+          amp_ops(nullptr),
+          effectComponents{{new Effect{this, FxSlot{0}},
+                            new Effect{this, FxSlot{1}},
+                            new Effect{this, FxSlot{2}},
+                            new Effect{this, FxSlot{3}},
+                            new Effect{this, FxSlot{4}},
+                            new Effect{this, FxSlot{5}},
+                            new Effect{this, FxSlot{6}},
+                            new Effect{this, FxSlot{7}}}}
     {
         ui->setupUi(this);
 
@@ -80,23 +99,28 @@ namespace plug
 
         // setting default values if there was none
         if (!settings.contains("Settings/connectOnStartup"))
+        {
             settings.setValue("Settings/connectOnStartup", true);
+        }
         if (!settings.contains("Settings/oneSetToSetThemAll"))
+        {
             settings.setValue("Settings/oneSetToSetThemAll", false);
+        }
         if (!settings.contains("Settings/keepWindowsOpen"))
+        {
             settings.setValue("Settings/keepWindowsOpen", false);
+        }
         if (!settings.contains("Settings/popupChangedWindows"))
+        {
             settings.setValue("Settings/popupChangedWindows", true);
+        }
         if (!settings.contains("Settings/defaultEffectValues"))
+        {
             settings.setValue("Settings/defaultEffectValues", true);
+        }
 
         // create child objects
         amp = new Amplifier(this);
-        effect1 = new Effect(this, 0);
-        effect2 = new Effect(this, 1);
-        effect3 = new Effect(this, 2);
-        effect4 = new Effect(this, 3);
-
         save = new SaveOnAmp(this);
         load = new LoadFromAmp(this);
         seffects = new SaveEffects(this);
@@ -108,10 +132,14 @@ namespace plug
 
         // connect buttons to slots
         connect(ui->Amplifier, SIGNAL(clicked()), amp, SLOT(showAndActivate()));
-        connect(ui->EffectButton1, SIGNAL(clicked()), effect1, SLOT(showAndActivate()));
-        connect(ui->EffectButton2, SIGNAL(clicked()), effect2, SLOT(showAndActivate()));
-        connect(ui->EffectButton3, SIGNAL(clicked()), effect3, SLOT(showAndActivate()));
-        connect(ui->EffectButton4, SIGNAL(clicked()), effect4, SLOT(showAndActivate()));
+        connect(ui->EffectButton1, SIGNAL(clicked()), effectComponents[0], SLOT(showAndActivate()));
+        connect(ui->EffectButton2, SIGNAL(clicked()), effectComponents[1], SLOT(showAndActivate()));
+        connect(ui->EffectButton3, SIGNAL(clicked()), effectComponents[2], SLOT(showAndActivate()));
+        connect(ui->EffectButton4, SIGNAL(clicked()), effectComponents[3], SLOT(showAndActivate()));
+        connect(ui->FxEffectButton1, SIGNAL(clicked()), effectComponents[4], SLOT(showAndActivate()));
+        connect(ui->FxEffectButton2, SIGNAL(clicked()), effectComponents[5], SLOT(showAndActivate()));
+        connect(ui->FxEffectButton3, SIGNAL(clicked()), effectComponents[6], SLOT(showAndActivate()));
+        connect(ui->FxEffectButton4, SIGNAL(clicked()), effectComponents[7], SLOT(showAndActivate()));
         connect(ui->actionConnect, SIGNAL(triggered()), this, SLOT(start_amp()));
         connect(ui->actionDisconnect, SIGNAL(triggered()), this, SLOT(stop_amp()));
         connect(ui->actionExit, SIGNAL(triggered()), this, SLOT(close()));
@@ -128,15 +156,32 @@ namespace plug
         connect(ui->action_Quick_presets, SIGNAL(triggered()), quickpres, SLOT(show()));
 
         // shortcuts to activate effect windows
-        QShortcut* showfx1 = new QShortcut(QKeySequence(Qt::CTRL + Qt::Key_1), this, nullptr, nullptr, Qt::ApplicationShortcut);
-        QShortcut* showfx2 = new QShortcut(QKeySequence(Qt::CTRL + Qt::Key_2), this, nullptr, nullptr, Qt::ApplicationShortcut);
-        QShortcut* showfx3 = new QShortcut(QKeySequence(Qt::CTRL + Qt::Key_3), this, nullptr, nullptr, Qt::ApplicationShortcut);
-        QShortcut* showfx4 = new QShortcut(QKeySequence(Qt::CTRL + Qt::Key_4), this, nullptr, nullptr, Qt::ApplicationShortcut);
-        QShortcut* showamp = new QShortcut(QKeySequence(Qt::CTRL + Qt::Key_5), this, nullptr, nullptr, Qt::ApplicationShortcut);
-        connect(showfx1, SIGNAL(activated()), this, SLOT(show_fx1()));
-        connect(showfx2, SIGNAL(activated()), this, SLOT(show_fx2()));
-        connect(showfx3, SIGNAL(activated()), this, SLOT(show_fx3()));
-        connect(showfx4, SIGNAL(activated()), this, SLOT(show_fx4()));
+        QShortcut* showFx1 = new QShortcut(QKeySequence(Qt::CTRL + Qt::Key_1), this, nullptr, nullptr, Qt::ApplicationShortcut);
+        QShortcut* showFx2 = new QShortcut(QKeySequence(Qt::CTRL + Qt::Key_2), this, nullptr, nullptr, Qt::ApplicationShortcut);
+        QShortcut* showFx3 = new QShortcut(QKeySequence(Qt::CTRL + Qt::Key_3), this, nullptr, nullptr, Qt::ApplicationShortcut);
+        QShortcut* showFx4 = new QShortcut(QKeySequence(Qt::CTRL + Qt::Key_4), this, nullptr, nullptr, Qt::ApplicationShortcut);
+        QShortcut* showFx5 = new QShortcut(QKeySequence(Qt::CTRL + Qt::Key_5), this, nullptr, nullptr, Qt::ApplicationShortcut);
+        QShortcut* showFx6 = new QShortcut(QKeySequence(Qt::CTRL + Qt::Key_6), this, nullptr, nullptr, Qt::ApplicationShortcut);
+        QShortcut* showFx7 = new QShortcut(QKeySequence(Qt::CTRL + Qt::Key_7), this, nullptr, nullptr, Qt::ApplicationShortcut);
+        QShortcut* showFx8 = new QShortcut(QKeySequence(Qt::CTRL + Qt::Key_8), this, nullptr, nullptr, Qt::ApplicationShortcut);
+        connect(showFx1, &QShortcut::activated, this, [this]
+                { this->showEffect(0); });
+        connect(showFx2, &QShortcut::activated, this, [this]
+                { this->showEffect(1); });
+        connect(showFx3, &QShortcut::activated, this, [this]
+                { this->showEffect(2); });
+        connect(showFx4, &QShortcut::activated, this, [this]
+                { this->showEffect(3); });
+        connect(showFx5, &QShortcut::activated, this, [this]
+                { this->showEffect(4); });
+        connect(showFx6, &QShortcut::activated, this, [this]
+                { this->showEffect(5); });
+        connect(showFx7, &QShortcut::activated, this, [this]
+                { this->showEffect(6); });
+        connect(showFx8, &QShortcut::activated, this, [this]
+                { this->showEffect(7); });
+
+        QShortcut* showamp = new QShortcut(QKeySequence(Qt::CTRL + Qt::Key_A), this, nullptr, nullptr, Qt::ApplicationShortcut);
         connect(showamp, SIGNAL(activated()), this, SLOT(show_amp()));
 
         // shortcuts for quick loading presets
@@ -150,16 +195,26 @@ namespace plug
         QShortcut* loadpres7 = new QShortcut(QKeySequence(Qt::Key_7), this, nullptr, nullptr, Qt::ApplicationShortcut);
         QShortcut* loadpres8 = new QShortcut(QKeySequence(Qt::Key_8), this, nullptr, nullptr, Qt::ApplicationShortcut);
         QShortcut* loadpres9 = new QShortcut(QKeySequence(Qt::Key_9), this, nullptr, nullptr, Qt::ApplicationShortcut);
-        connect(loadpres0, SIGNAL(activated()), this, SLOT(load_presets0()));
-        connect(loadpres1, SIGNAL(activated()), this, SLOT(load_presets1()));
-        connect(loadpres2, SIGNAL(activated()), this, SLOT(load_presets2()));
-        connect(loadpres3, SIGNAL(activated()), this, SLOT(load_presets3()));
-        connect(loadpres4, SIGNAL(activated()), this, SLOT(load_presets4()));
-        connect(loadpres5, SIGNAL(activated()), this, SLOT(load_presets5()));
-        connect(loadpres6, SIGNAL(activated()), this, SLOT(load_presets6()));
-        connect(loadpres7, SIGNAL(activated()), this, SLOT(load_presets7()));
-        connect(loadpres8, SIGNAL(activated()), this, SLOT(load_presets8()));
-        connect(loadpres9, SIGNAL(activated()), this, SLOT(load_presets9()));
+        connect(loadpres0, &QShortcut::activated, this, [this]
+                { loadPreset(0); });
+        connect(loadpres1, &QShortcut::activated, this, [this]
+                { loadPreset(1); });
+        connect(loadpres2, &QShortcut::activated, this, [this]
+                { loadPreset(2); });
+        connect(loadpres3, &QShortcut::activated, this, [this]
+                { loadPreset(3); });
+        connect(loadpres4, &QShortcut::activated, this, [this]
+                { loadPreset(4); });
+        connect(loadpres5, &QShortcut::activated, this, [this]
+                { loadPreset(5); });
+        connect(loadpres6, &QShortcut::activated, this, [this]
+                { loadPreset(6); });
+        connect(loadpres7, &QShortcut::activated, this, [this]
+                { loadPreset(7); });
+        connect(loadpres8, &QShortcut::activated, this, [this]
+                { loadPreset(8); });
+        connect(loadpres9, &QShortcut::activated, this, [this]
+                { loadPreset(9); });
 
         // shortcut to activate buttons
         QShortcut* shortcut = new QShortcut(QKeySequence(Qt::CTRL + Qt::SHIFT + Qt::Key_A), this);
@@ -193,7 +248,7 @@ namespace plug
                               "<p>%1 is a GPLv3 licensed program designed as a replacement for Fender FUSE to operate Fender Mustang amplifier and possibly some other models.</p>"
                               "<p><a href=\"https://github.com/offa/plug/\">Plug (GitHub)</a> / <a href=\"https://gitlab.com/offa/plug/\">Plug (GitLab)</a></p>"
                               "<hr>"
-                              "<p>Copyright © 2017-2021 offa</p>"
+                              "<p>Copyright © 2017-2023 offa</p>"
                               "<p>Copyright © 2010-2016 piorekf <piorek@piorekf.org></p>"
                               "<p>License: <a href=\"https://www.gnu.org/licenses/gpl.txt\">GPLv3</a></p>"
                               "</center>")
@@ -208,7 +263,7 @@ namespace plug
     {
         QSettings settings;
         amp_settings amplifier_set{};
-        std::array<fx_pedal_settings, 4> effects_set{{}};
+        std::vector<fx_pedal_settings> effects_set{};
         QString name;
 
         ui->statusBar->showMessage(tr("Connecting..."));
@@ -216,7 +271,7 @@ namespace plug
 
         try
         {
-            amp_ops = std::make_unique<plug::com::Mustang>(plug::com::createUsbConnection());
+            amp_ops = plug::com::connect();
             const auto [signalChain, presets] = amp_ops->start_amp();
             name = QString::fromStdString(signalChain.name());
             amplifier_set = signalChain.amp();
@@ -241,7 +296,11 @@ namespace plug
         }
         else
         {
-            setWindowTitle(QString(tr("PLUG: %1")).arg(name));
+            const auto model = amp_ops->getDeviceModel();
+            setWindowTitle(QString(tr("PLUG - %1 %2: %3"))
+                               .arg(QString::fromStdString(model.name()))
+                               .arg(model.category() == DeviceModel::Category::MustangV2 ? "(v2)" : "")
+                               .arg(name));
             setAccessibleName(QString(tr("Main window: %1")).arg(name));
         }
 
@@ -252,50 +311,20 @@ namespace plug
         {
             amp->show();
         }
-        for (std::size_t i = 0; i < 4; i++)
-        {
-            switch (effects_set[i].fx_slot)
+
+        std::for_each(effects_set.cbegin(), effects_set.cend(), [this, &settings](const auto& effect)
+                      {
+            Effect* component = effectComponents.at(effect.slot.id());
+            component->load(effect);
+
+            if ((effect.effect_num != effects::EMPTY) && (settings.value("Settings/popupChangedWindows").toBool()))
             {
-                case 0x00:
-                case 0x04:
-                    effect1->load(effects_set[i]);
-                    if (effects_set[i].effect_num != effects::EMPTY)
-                        if (settings.value("Settings/popupChangedWindows").toBool())
-                            effect1->show();
-                    break;
-
-                case 0x01:
-                case 0x05:
-                    effect2->load(effects_set[i]);
-                    if (effects_set[i].effect_num != effects::EMPTY)
-                        if (settings.value("Settings/popupChangedWindows").toBool())
-                            effect2->show();
-                    break;
-
-                case 0x02:
-                case 0x06:
-                    effect3->load(effects_set[i]);
-                    if (effects_set[i].effect_num != effects::EMPTY)
-                        if (settings.value("Settings/popupChangedWindows").toBool())
-                            effect3->show();
-                    break;
-
-                case 0x03:
-                case 0x07:
-                    effect4->load(effects_set[i]);
-                    if (effects_set[i].effect_num != effects::EMPTY)
-                        if (settings.value("Settings/popupChangedWindows").toBool())
-                            effect4->show();
-                    break;
-            }
-        }
-
+                component->show();
+            } });
         // activate buttons
         amp->enable_set_button(true);
-        effect1->enable_set_button(true);
-        effect2->enable_set_button(true);
-        effect3->enable_set_button(true);
-        effect4->enable_set_button(true);
+        std::for_each(effectComponents.cbegin(), effectComponents.cend(), [](const auto& effect)
+                      { effect->enable_set_button(true); });
         ui->actionConnect->setDisabled(true);
         ui->actionDisconnect->setDisabled(false);
         ui->actionSave_to_amplifier->setDisabled(false);
@@ -319,10 +348,8 @@ namespace plug
 
             // deactivate buttons
             amp->enable_set_button(false);
-            effect1->enable_set_button(false);
-            effect2->enable_set_button(false);
-            effect3->enable_set_button(false);
-            effect4->enable_set_button(false);
+            std::for_each(effectComponents.cbegin(), effectComponents.cend(), [](const auto& effect)
+                          { effect->enable_set_button(false); });
             ui->actionConnect->setDisabled(false);
             ui->actionDisconnect->setDisabled(true);
             ui->actionSave_to_amplifier->setDisabled(true);
@@ -371,7 +398,6 @@ namespace plug
 
     void MainWindow::set_amplifier(amp_settings amp_settings)
     {
-
         if (!connected)
         {
             return;
@@ -383,28 +409,12 @@ namespace plug
         {
             if (settings.value("Settings/oneSetToSetThemAll").toBool())
             {
-                fx_pedal_settings pedal{};
-
-                if (effect1->get_changed())
-                {
-                    effect1->get_settings(pedal);
-                    amp_ops->set_effect(pedal);
-                }
-                if (effect2->get_changed())
-                {
-                    effect2->get_settings(pedal);
-                    amp_ops->set_effect(pedal);
-                }
-                if (effect3->get_changed())
-                {
-                    effect3->get_settings(pedal);
-                    amp_ops->set_effect(pedal);
-                }
-                if (effect4->get_changed())
-                {
-                    effect4->get_settings(pedal);
-                    amp_ops->set_effect(pedal);
-                }
+                std::for_each(effectComponents.begin(), effectComponents.end(), [this](const auto& comp)
+                              {
+                    if (comp->get_changed())
+                    {
+                        amp_ops->set_effect(comp->getSettings());
+                    } });
             }
 
             amp_ops->set_amplifier(amp_settings);
@@ -484,47 +494,16 @@ namespace plug
             }
 
             const auto effects_set = signalChain.effects();
-            for (std::size_t i = 0; i < 4; i++)
-            {
-                switch (effects_set[i].fx_slot)
+            const bool shouldPopup = settings.value("Settings/popupChangedWindows").toBool();
+            std::for_each(effects_set.cbegin(), effects_set.cend(), [this, shouldPopup](const auto& effect)
+                          {
+                const auto component = effectComponents.at(effect.slot.id());
+
+                component->load(effect);
+                if ((effect.effect_num != effects::EMPTY) && shouldPopup)
                 {
-                    case 0x00:
-                    case 0x04:
-                        effect1->load(effects_set[i]);
-                        if (effects_set[i].effect_num != effects::EMPTY)
-                            if (settings.value("Settings/popupChangedWindows").toBool())
-                                effect1->show();
-                        break;
-
-                    case 0x01:
-                    case 0x05:
-                        effect2->load(effects_set[i]);
-                        if (effects_set[i].effect_num != effects::EMPTY)
-                            if (settings.value("Settings/popupChangedWindows").toBool())
-                                effect2->show();
-                        break;
-
-                    case 0x02:
-                    case 0x06:
-                        effect3->load(effects_set[i]);
-                        if (effects_set[i].effect_num != effects::EMPTY)
-                            if (settings.value("Settings/popupChangedWindows").toBool())
-                                effect3->show();
-                        break;
-
-                    case 0x03:
-                    case 0x07:
-                        effect4->load(effects_set[i]);
-                        if (effects_set[i].effect_num != effects::EMPTY)
-                        {
-                            if (settings.value("Settings/popupChangedWindows").toBool())
-                            {
-                                effect4->show();
-                            }
-                        }
-                        break;
-                }
-            }
+                    component->show();
+                } });
         }
         catch (const std::exception& ex)
         {
@@ -538,10 +517,8 @@ namespace plug
     void MainWindow::enable_buttons()
     {
         amp->enable_set_button(true);
-        effect1->enable_set_button(true);
-        effect2->enable_set_button(true);
-        effect3->enable_set_button(true);
-        effect4->enable_set_button(true);
+        std::for_each(effectComponents.cbegin(), effectComponents.cend(), [](const auto& effect)
+                      { effect->enable_set_button(true); });
         ui->actionConnect->setDisabled(false);
         ui->actionDisconnect->setDisabled(false);
         ui->actionSave_to_amplifier->setDisabled(false);
@@ -563,23 +540,23 @@ namespace plug
 
     void MainWindow::save_effects(int slot, char* name, int fx_num, bool mod, bool dly, bool rev)
     {
-        std::vector<fx_pedal_settings> effects(static_cast<std::size_t>(fx_num));
+        std::vector<fx_pedal_settings> effects(static_cast<std::size_t>(fx_num), {FxSlot{0}, effects::EMPTY, 0, 0, 0, 0, 0, 0, false});
 
         if (fx_num == 1)
         {
             if (mod)
             {
-                effect2->get_settings(effects[0]);
+                effects[0] = effectComponents[1]->getSettings();
                 set_effect(effects[0]);
             }
             else if (dly)
             {
-                effect3->get_settings(effects[0]);
+                effects[0] = effectComponents[2]->getSettings();
                 set_effect(effects[0]);
             }
             else if (rev)
             {
-                effect4->get_settings(effects[0]);
+                effects[0] = effectComponents[3]->getSettings();
                 set_effect(effects[0]);
             }
             else
@@ -589,9 +566,9 @@ namespace plug
         }
         else
         {
-            effect3->get_settings(effects[0]);
+            effects[0] = effectComponents[2]->getSettings();
             set_effect(effects[0]);
-            effect4->get_settings(effects[1]);
+            effects[1] = effectComponents[3]->getSettings();
             set_effect(effects[1]);
         }
 
@@ -622,11 +599,11 @@ namespace plug
         }
 
         settings.setValue("LoadFile/lastDirectory", QFileInfo(filename).absolutePath());
-        auto file = std::make_unique<QFile>(filename, this);
+        QFile file{filename, this};
 
-        if (file->exists())
+        if (file.exists())
         {
-            if (!file->open(QFile::ReadOnly | QFile::Text))
+            if (!file.open(QFile::ReadOnly | QFile::Text))
             {
                 QMessageBox::critical(this, tr("Error!"), tr("Could not open file"));
                 return;
@@ -638,81 +615,52 @@ namespace plug
             return;
         }
 
-        amp_settings amplifier_set{};
-        fx_pedal_settings effects_set[4];
-        QString name;
-        auto loader = std::make_unique<LoadFromFile>(file.get(), &name, &amplifier_set, effects_set);
-        loader->loadfile();
-        file->close();
+        LoadFromFile loader{&file};
+        const auto fileSettings = loader.loadfile();
+        file.close();
 
-        change_title(name);
+        change_title(fileSettings.name);
 
-        amp->load(amplifier_set);
+        amp->load(fileSettings.amp);
         if (connected)
         {
             amp->send_amp();
         }
-        if (settings.value("Settings/popupChangedWindows").toBool())
+
+        const bool shouldPopup = settings.value("Settings/popupChangedWindows").toBool();
+
+        if (shouldPopup)
         {
             amp->show();
         }
 
-        for (int i = 0; i < 4; i++)
-        {
-            switch (effects_set[i].fx_slot)
+        std::for_each(fileSettings.effects.cbegin(), fileSettings.effects.cend(), [this, shouldPopup](auto& effect)
+                      {
+            const auto& component = effectComponents.at(effect.slot.id());
+            component->load(effect);
+
+            if (connected)
             {
-                case 0x00:
-                    effect1->load(effects_set[i]);
-                    if (connected)
-                        effect1->send_fx();
-                    if (effects_set[i].effect_num != effects::EMPTY)
-                        if (settings.value("Settings/popupChangedWindows").toBool())
-                            effect1->show();
-                    break;
-
-                case 0x01:
-                    effect2->load(effects_set[i]);
-                    if (connected)
-                        effect2->send_fx();
-                    if (effects_set[i].effect_num != effects::EMPTY)
-                        if (settings.value("Settings/popupChangedWindows").toBool())
-                            effect2->show();
-                    break;
-
-                case 0x02:
-                    effect3->load(effects_set[i]);
-                    if (connected)
-                        effect3->send_fx();
-                    if (effects_set[i].effect_num != effects::EMPTY)
-                        if (settings.value("Settings/popupChangedWindows").toBool())
-                            effect3->show();
-                    break;
-
-                case 0x03:
-                    effect4->load(effects_set[i]);
-                    if (connected)
-                        effect4->send_fx();
-                    if (effects_set[i].effect_num != effects::EMPTY)
-                        if (settings.value("Settings/popupChangedWindows").toBool())
-                            effect4->show();
-                    break;
+                component->send_fx();
             }
-        }
+
+            if ((effect.effect_num != effects::EMPTY) && shouldPopup)
+            {
+                component->show();
+            } });
     }
 
-    void MainWindow::get_settings(amp_settings* amplifier_settings, fx_pedal_settings fx_settings[4])
+    void MainWindow::get_settings(amp_settings* amplifier_settings, std::vector<fx_pedal_settings>& fx_settings)
     {
         if (amplifier_settings != nullptr)
         {
             amp->get_settings(amplifier_settings);
         }
-        if (fx_settings != nullptr)
-        {
-            effect1->get_settings(fx_settings[0]);
-            effect2->get_settings(fx_settings[1]);
-            effect3->get_settings(fx_settings[2]);
-            effect4->get_settings(fx_settings[3]);
-        }
+
+        fx_settings = std::vector<fx_pedal_settings>{};
+
+        std::for_each(effectComponents.cbegin(), effectComponents.cend(), [&fx_settings](const auto& comp)
+                      { fx_settings.push_back(comp->getSettings()); });
     }
 
     void MainWindow::change_title(const QString& name)
@@ -731,39 +679,17 @@ namespace plug
         }
     }
 
-    void MainWindow::show_fx1()
+    void MainWindow::showEffect(std::uint8_t slot)
     {
-        if (!effect1->isVisible())
+        auto comp = effectComponents.at(slot);
+
+        if (!comp->isVisible())
         {
-            effect1->show();
+            comp->show();
         }
-        effect1->activateWindow();
+        comp->activateWindow();
     }
 
-    void MainWindow::show_fx2()
-    {
-        if (!effect2->isVisible())
-        {
-            effect2->show();
-        }
-        effect2->activateWindow();
-    }
-    void MainWindow::show_fx3()
-    {
-        if (!effect3->isVisible())
-        {
-            effect3->show();
-        }
-        effect3->activateWindow();
-    }
-    void MainWindow::show_fx4()
-    {
-        if (!effect4->isVisible())
-        {
-            effect4->show();
-        }
-        effect4->activateWindow();
-    }
 
     void MainWindow::show_amp()
     {
@@ -781,14 +707,12 @@ namespace plug
 
         settings.setValue("Settings/popupChangedWindows", false);
 
-        library = std::make_unique<Library>(presetNames, this);
-        effect1->close();
-        effect2->close();
-        effect3->close();
-        effect4->close();
+        Library library{presetNames, this};
+        std::for_each(effectComponents.cbegin(), effectComponents.cend(), [](const auto& comp)
+                      { comp->close(); });
         amp->close();
         this->close();
-        library->exec();
+        library.exec();
 
         settings.setValue("Settings/popupChangedWindows", previous);
         this->show();
@@ -797,8 +721,6 @@ namespace plug
     void MainWindow::update_firmware()
     {
         QString filename;
-        int ret = 0;
-
         QMessageBox::information(this, "Prepare", R"(Please power off the amplifier, then power it back on while holding down:<ul><li>The "Save" button (Mustang I and II)</li><li>The Data Wheel (Mustang III, IV and IV)</li></ul>After pressing "OK" choose firmware file and then update will begin.It will take about one minute. You will be notified when it's finished.)");
 
         filename = QFileDialog::getOpenFileName(this, tr("Open..."), QDir::homePath(), tr("Mustang firmware (*.upd)"));
@@ -816,7 +738,7 @@ namespace plug
         ui->centralWidget->setDisabled(true);
         ui->menuBar->setDisabled(true);
         this->repaint();
-        ret = com::updateFirmware(filename.toLatin1().constData());
+        const int ret = com::updateFirmware(filename.toLatin1().constData());
         ui->centralWidget->setDisabled(false);
         ui->menuBar->setDisabled(false);
         ui->statusBar->showMessage("", 1);
@@ -835,136 +757,39 @@ namespace plug
 
     void MainWindow::show_default_effects()
     {
-        deffx = std::make_unique<DefaultEffects>(this);
-        deffx->exec();
-        deffx.reset();
+        DefaultEffects deffx{this};
+        deffx.exec();
     }
 
     void MainWindow::empty_other(int value, Effect* caller)
     {
         const int fx_family = check_fx_family(static_cast<effects>(value));
-        fx_pedal_settings settings{};
+        fx_pedal_settings settings{FxSlot{0}, effects::EMPTY, 0, 0, 0, 0, 0, 0, false};
 
-        if (caller != effect1)
-        {
-            effect1->get_settings(settings);
-            if (check_fx_family(settings.effect_num) == fx_family)
+        std::for_each(effectComponents.cbegin(), effectComponents.cend(), [&caller, &settings, fx_family](const auto& comp)
+                      {
+            if (caller != comp)
             {
-                effect1->choose_fx(0);
-                effect1->send_fx();
-            }
-        }
+                settings = comp->getSettings();
 
-        if (caller != effect2)
-        {
-            effect2->get_settings(settings);
-            if (check_fx_family(settings.effect_num) == fx_family)
-            {
-                effect2->choose_fx(0);
-                effect2->send_fx();
-            }
-        }
+                if (check_fx_family(settings.effect_num) == fx_family)
+                {
+                    comp->choose_fx(0);
+                    comp->send_fx();
+                }
+            } });
+    }
 
-        if (caller != effect3)
-        {
-            effect3->get_settings(settings);
-            if (check_fx_family(settings.effect_num) == fx_family)
-            {
-                effect3->choose_fx(0);
-                effect3->send_fx();
-            }
-        }
+    void MainWindow::loadPreset(std::size_t number)
+    {
+        QSettings settings;
 
-        if (caller != effect4)
+        if (const auto key = QString{"DefaultPresets/Preset%1"}.arg(number); settings.contains(key))
         {
-            effect4->get_settings(settings);
-            if (check_fx_family(settings.effect_num) == fx_family)
-            {
-                effect4->choose_fx(0);
-                effect4->send_fx();
-            }
+            load_from_amp(settings.value(key).toInt());
         }
     }
 
-    void MainWindow::load_presets0()
-    {
-        QSettings settings;
-
-        if (settings.contains("DefaultPresets/Preset0"))
-            load_from_amp(settings.value("DefaultPresets/Preset0").toInt());
-    }
-
-    void MainWindow::load_presets1()
-    {
-        QSettings settings;
-
-        if (settings.contains("DefaultPresets/Preset1"))
-            load_from_amp(settings.value("DefaultPresets/Preset1").toInt());
-    }
-
-    void MainWindow::load_presets2()
-    {
-        QSettings settings;
-
-        if (settings.contains("DefaultPresets/Preset2"))
-            load_from_amp(settings.value("DefaultPresets/Preset2").toInt());
-    }
-
-    void MainWindow::load_presets3()
-    {
-        QSettings settings;
-
-        if (settings.contains("DefaultPresets/Preset3"))
-            load_from_amp(settings.value("DefaultPresets/Preset3").toInt());
-    }
-
-    void MainWindow::load_presets4()
-    {
-        QSettings settings;
-
-        if (settings.contains("DefaultPresets/Preset4"))
-            load_from_amp(settings.value("DefaultPresets/Preset4").toInt());
-    }
-
-    void MainWindow::load_presets5()
-    {
-        QSettings settings;
-
-        if (settings.contains("DefaultPresets/Preset5"))
-            load_from_amp(settings.value("DefaultPresets/Preset5").toInt());
-    }
-
-    void MainWindow::load_presets6()
-    {
-        QSettings settings;
-
-        if (settings.contains("DefaultPresets/Preset6"))
-            load_from_amp(settings.value("DefaultPresets/Preset6").toInt());
-    }
-
-    void MainWindow::load_presets7()
-    {
-        QSettings settings;
-
-        if (settings.contains("DefaultPresets/Preset7"))
-            load_from_amp(settings.value("DefaultPresets/Preset7").toInt());
-    }
-
-    void MainWindow::load_presets8()
-    {
-        QSettings settings;
-
-        if (settings.contains("DefaultPresets/Preset8"))
-            load_from_amp(settings.value("DefaultPresets/Preset8").toInt());
-    }
-
-    void MainWindow::load_presets9()
-    {
-        QSettings settings;
-
-        if (settings.contains("DefaultPresets/Preset9"))
-            load_from_amp(settings.value("DefaultPresets/Preset9").toInt());
-    }
 }
 
 #include "ui/moc_mainwindow.moc"
