@@ -2,7 +2,7 @@
  * PLUG - software to operate Fender Mustang amplifier
  *        Linux replacement for Fender FUSE software
  *
- * Copyright (C) 2017-2022  offa
+ * Copyright (C) 2017-2023  offa
  * Copyright (C) 2010-2016  piorekf <piorek@piorekf.org>
  *
  * This program is free software: you can redistribute it and/or modify
@@ -22,6 +22,8 @@
 #include "ui/savetofile.h"
 #include "ui/mainwindow.h"
 #include "ui_savetofile.h"
+#include <QFileDialog>
+#include <QMessageBox>
 
 namespace plug
 {
@@ -70,7 +72,7 @@ namespace plug
 
         xml = std::make_unique<QXmlStreamWriter>(file.get());
         amp_settings amplifier_settings{};
-        fx_pedal_settings fx_settings[4];
+        std::vector<fx_pedal_settings> fx_settings{};
         dynamic_cast<MainWindow*>(parent())->get_settings(&amplifier_settings, fx_settings);
 
         xml->setAutoFormatting(true);
@@ -301,108 +303,45 @@ namespace plug
         xml->writeEndElement(); // end Module
     }
 
-    void SaveToFile::manageWriteFX(fx_pedal_settings settings[4])
+    void SaveToFile::manageWriteFX(const std::vector<fx_pedal_settings>& settings)
     {
-        fx_pedal_settings empty{0, effects::EMPTY, 0, 0, 0, 0, 0, 0, Position::input};
-
+        constexpr fx_pedal_settings empty{FxSlot{0}, effects::EMPTY, 0, 0, 0, 0, 0, 0, false};
+        auto writeOrEmpty = [&settings, empty, this](const auto& itr)
+        { writeFX(itr != settings.cend() ? *itr : empty); };
         xml->writeStartElement("FX");
 
+
         xml->writeStartElement("Stompbox");
         xml->writeAttribute("ID", "1");
-        if ((settings[0].effect_num > effects::EMPTY) && (settings[0].effect_num <= effects::COMPRESSOR))
-        {
-            writeFX(settings[0]);
-        }
-        else if ((settings[1].effect_num > effects::EMPTY) && (settings[1].effect_num <= effects::COMPRESSOR))
-        {
-            writeFX(settings[1]);
-        }
-        else if ((settings[2].effect_num > effects::EMPTY) && (settings[2].effect_num <= effects::COMPRESSOR))
-        {
-            writeFX(settings[2]);
-        }
-        else if ((settings[3].effect_num > effects::EMPTY) && (settings[3].effect_num <= effects::COMPRESSOR))
-        {
-            writeFX(settings[3]);
-        }
-        else
-        {
-            writeFX(empty);
-        }
+
+        auto stomp = std::find_if(settings.cbegin(), settings.cend(), [](const auto& effect)
+                                  { return (effect.effect_num > effects::EMPTY) && (effect.effect_num <= effects::COMPRESSOR); });
+        writeOrEmpty(stomp);
         xml->writeEndElement(); // end Stompbox
 
+
         xml->writeStartElement("Modulation");
         xml->writeAttribute("ID", "2");
-        if ((settings[0].effect_num >= effects::SINE_CHORUS) && (settings[0].effect_num <= effects::PITCH_SHIFTER))
-        {
-            writeFX(settings[0]);
-        }
-        else if ((settings[1].effect_num >= effects::SINE_CHORUS) && (settings[1].effect_num <= effects::PITCH_SHIFTER))
-        {
-            writeFX(settings[1]);
-        }
-        else if ((settings[2].effect_num >= effects::SINE_CHORUS) && (settings[2].effect_num <= effects::PITCH_SHIFTER))
-        {
-            writeFX(settings[2]);
-        }
-        else if ((settings[3].effect_num >= effects::SINE_CHORUS) && (settings[3].effect_num <= effects::PITCH_SHIFTER))
-        {
-            writeFX(settings[3]);
-        }
-        else
-        {
-            writeFX(empty);
-        }
+        auto modulation = std::find_if(settings.cbegin(), settings.cend(), [](const auto& effect)
+                                       { return (effect.effect_num >= effects::SINE_CHORUS) && (effect.effect_num <= effects::PITCH_SHIFTER); });
+        writeOrEmpty(modulation);
         xml->writeEndElement(); // end Modulation
 
+
         xml->writeStartElement("Delay");
         xml->writeAttribute("ID", "3");
-        if ((settings[0].effect_num >= effects::MONO_DELAY) && (settings[0].effect_num <= effects::STEREO_TAPE_DELAY))
-        {
-            writeFX(settings[0]);
-        }
-        else if ((settings[1].effect_num >= effects::MONO_DELAY) && (settings[1].effect_num <= effects::STEREO_TAPE_DELAY))
-        {
-            writeFX(settings[1]);
-        }
-        else if ((settings[2].effect_num >= effects::MONO_DELAY) && (settings[2].effect_num <= effects::STEREO_TAPE_DELAY))
-        {
-            writeFX(settings[2]);
-        }
-        else if ((settings[3].effect_num >= effects::MONO_DELAY) && (settings[3].effect_num <= effects::STEREO_TAPE_DELAY))
-        {
-            writeFX(settings[3]);
-        }
-        else
-        {
-            writeFX(empty);
-        }
+        auto delay = std::find_if(settings.cbegin(), settings.cend(), [](const auto& effect)
+                                  { return (effect.effect_num >= effects::MONO_DELAY) && (effect.effect_num <= effects::STEREO_TAPE_DELAY); });
+        writeOrEmpty(delay);
         xml->writeEndElement(); // end Delay
 
+
         xml->writeStartElement("Reverb");
         xml->writeAttribute("ID", "4");
-        if ((settings[0].effect_num >= effects::SMALL_HALL_REVERB) && (settings[0].effect_num <= effects::FENDER_65_SPRING_REVERB))
-        {
-            writeFX(settings[0]);
-        }
-        else if ((settings[1].effect_num >= effects::SMALL_HALL_REVERB) && (settings[1].effect_num <= effects::FENDER_65_SPRING_REVERB))
-        {
-            writeFX(settings[1]);
-        }
-        else if ((settings[2].effect_num >= effects::SMALL_HALL_REVERB) && (settings[2].effect_num <= effects::FENDER_65_SPRING_REVERB))
-        {
-            writeFX(settings[2]);
-        }
-        else if ((settings[3].effect_num >= effects::SMALL_HALL_REVERB) && (settings[3].effect_num <= effects::FENDER_65_SPRING_REVERB))
-        {
-            writeFX(settings[3]);
-        }
-        else
-        {
-            writeFX(empty);
-        }
+        auto reverb = std::find_if(settings.cbegin(), settings.cend(), [](const auto& effect)
+                                   { return (effect.effect_num >= effects::SMALL_HALL_REVERB) && (effect.effect_num <= effects::FENDER_65_SPRING_REVERB); });
+        writeOrEmpty(reverb);
         xml->writeEndElement(); // end Reverb
-
         xml->writeEndElement(); // end FX
     }
 
@@ -565,11 +504,9 @@ namespace plug
                 break;
         }
 
-        const int position = (settings.position == Position::effectsLoop ? (settings.fx_slot + 4) : settings.fx_slot);
-
         xml->writeStartElement("Module");
         xml->writeAttribute("ID", QString("%1").arg(model));
-        xml->writeAttribute("POS", QString("%1").arg(position));
+        xml->writeAttribute("POS", QString("%1").arg(settings.slot.id()));
         xml->writeAttribute("BypassState", "1");
 
         if (model == value(effects::EMPTY))
