@@ -2,7 +2,7 @@
  * PLUG - software to operate Fender Mustang amplifier
  *        Linux replacement for Fender FUSE software
  *
- * Copyright (C) 2017-2022  offa
+ * Copyright (C) 2017-2023  offa
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -23,7 +23,7 @@
 #include "com/Connection.h"
 #include <gmock/gmock.h>
 
-namespace mock
+namespace plug::test::mock
 {
     class MockConnection : public plug::com::Connection
     {
@@ -33,10 +33,11 @@ namespace mock
         }
 
         MOCK_METHOD(void, open, (std::uint16_t, std::uint16_t));
-        MOCK_METHOD(void, openFirst, (std::uint16_t, std::initializer_list<std::uint16_t>));
+        MOCK_METHOD(void, openFirst, (std::uint16_t, std::initializer_list<std::uint16_t>) );
         MOCK_METHOD(void, close, ());
         MOCK_METHOD(bool, isOpen, (), (const));
         MOCK_METHOD(std::vector<std::uint8_t>, receive, (std::size_t));
         MOCK_METHOD(std::size_t, sendImpl, (std::uint8_t*, std::size_t));
+        MOCK_METHOD(std::string, name, (), (const));
     };
 }
